# frozen_string_literal: true

module  Apiphobic
class   AcceptHeader
  attr_accessor :raw_accept_header
  attr_reader   :application_name

  def initialize(header, application_name:)
    self.application_name  = application_name
    self.raw_accept_header = header || ''
  end

  def application_name=(other)
    @accept_header_data   = nil
    @accept_header_format = nil

    @application_name     = other
  end

  def version
    accept_header_data[2]
  end

  def content_type
    accept_header_data[1]
  end

  def valid?
    !invalid?
  end

  def invalid?
    accept_header_data.nil?
  end

  def to_s
    raw_accept_header
  end

  private

  def accept_header_data
    @accept_header_data ||= raw_accept_header.match(accept_header_format)
  end

  def accept_header_format
    @accept_header_format ||= %r{
      ################################################################################
      # BEGINNING
      ################################################################################

      (?:
        (?<=\A)                       # the beginning of the accept header
        |                             # or
        (?<=,)                        # the beginning of an accept value
      )

      ################################################################################
      # APPLICATION NAME
      ################################################################################

      application/vnd\.#{application_name} # the application vendor string

      ################################################################################
      # SUBRESOURCE
      ################################################################################

      (?:
        \+                            # a literal '+'
        (\w+)                         # the subresource name
      )?

      ################################################################################
      # VERSION SECTION
      ################################################################################

      (?:
        ;                             # a literal ';'

        version=                      # a literal 'version='
        (
                                      # VERSION DIGIT GROUPINGS SECTION

          \d+                         # one or more digits
          (?:\.\d+){0,2}              # one or two groups of digits with preceeding '.'

                                      # VERSION BETA SECTION
          (?:
            beta                      # a literal 'beta'
            (?:\d*)                   # a optional beta version
          )?
        )
      )?

      ################################################################################
      # END
      ################################################################################

      (?:
        (?=\z)                        # the end of the accept header
        |                             # or
        (?=,)                         # the end of an accept value
      )
    }x
  end
end
end
