# frozen_string_literal: true

require 'apiphobic/matchers/accept_header'
require 'apiphobic/matchers/subdomain'
require 'apiphobic/middleware/configurable'
require 'apiphobic/requests/accept_header'
require 'apiphobic/requests/subdomain'
require 'apiphobic/responses/invalid_accept_header'

module  Apiphobic
module  Middleware
module  Validators
class   AcceptHeader
  include Configurable

  def initialize(app)
    @app = app
  end

  def call(env)
    subdomain_request     = Requests::Subdomain.new(env)
    accept_header_request = Requests::AcceptHeader.new(env)
    accept_header         = Matchers::AcceptHeader.new(
                              application_name: configuration.application_name,
                            )
    subdomain             = Matchers::Subdomain.new(
                              allowed_subdomains: configuration.allowed_api_subdomains,
                            )

    if !subdomain.matches?(subdomain_request) ||
       accept_header.matches?(accept_header_request)
      @app.call(env)
    else
      Responses::InvalidAcceptHeader.call(env)
    end
  end
end
end
end
end
