# frozen_string_literal: true

require 'erratum/errors/authentication/invalid_token'
require 'apiphobic/requests/authorization_token'
require 'apiphobic/responses/invalid_token'
require 'apiphobic/tokens/configuration'

module  Apiphobic
module  Middleware
module  Validators
class   AuthorizationToken
  def initialize(app)
    @app = app
  end

  def call(env)
    token_request = Requests::AuthorizationToken.new(
                      env,
                      type:        Apiphobic::Tokens.configuration.type,
                      private_key: Apiphobic::Tokens.configuration.private_key,
                    )
    token         = token_request.fetch

    token.validate!

    env['X_JSON_WEB_TOKEN'] = token.to_h

    @app.call(env)
  rescue ::Erratum::Errors::InvalidToken => error
    Responses::InvalidToken.call(env, error: error)
  end
end
end
end
end
