# frozen_string_literal: true

module  Apiphobic
module  Responses
class   TransformJsonApi
  attr_accessor :status,
                :headers,
                :body

  def initialize(status, headers, body)
    self.status  = status
    self.headers = headers
    self.body    = body
  end

  def transform
    [
      status,
      transformed_headers,
      body,
    ]
  end

  private

  def transformed_headers
    headers['rack.input'] = StringIO.new(transformed_json_body)

    headers
  end

  def transformed_json_body
    return request_body if request_body == '' || request_body.nil?

    relationships = request_hash.dig('data', 'relationships') || {}

    relationships.dup.each do |relationship_name, data|
      relationships.delete(relationship_name)
      relationships[relationship_name.tr('_', '-')] = data
    end

    JSON.dump(request_hash)
  end

  def request_hash
    @request_hash ||= JSON.parse(request_body)
  end

  def request_body
    @request_body ||= headers['rack.input']&.read.to_s
  ensure
    headers['rack.input']&.rewind
  end
end
end
end
